package cn.wise.sc.pay.pass.business.cashiers.controller;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.merchant.service.IMerchantProfitService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.api.order.service.ISafeguardService;
import cn.wise.sc.pay.api.points.service.ICashPointRecordService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.order.OrderStatusUtils;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import cn.wise.sc.pay.pass.business.cashiers.service.IPaasOrderService;
import cn.wise.sc.payment.api.service.IZHPaymentService;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 * @description: 入驻商家订购单
 * @author: qh
 * @create: 2019-12-04 15:14
 **/
@Validated
@CrossOrigin
@RequestMapping("/cashier/order")
@RestController
@Api(description = "订单")
public class CashierOrderController {

    final
    IPaasOrderService iOrderService;
    final
    IOrdersService ordersService;
    final
    IMerchantProfitService merchantProfitService;
    final
    IZHPaymentService izhPaymentService;
    final
    ICashPointRecordService iCashPointRecordService;
    final
    ISafeguardService iSafeguardService;

    public CashierOrderController(IPaasOrderService iOrderService, IOrdersService ordersService, IMerchantProfitService merchantProfitService, IZHPaymentService izhPaymentService, ICashPointRecordService iCashPointRecordService, ISafeguardService iSafeguardService) {
        this.iOrderService = iOrderService;
        this.ordersService = ordersService;
        this.merchantProfitService = merchantProfitService;
        this.izhPaymentService = izhPaymentService;
        this.iCashPointRecordService = iCashPointRecordService;
        this.iSafeguardService = iSafeguardService;
    }

    @PostMapping("/new")
    @ApiOperation("新增一个订单")
    @ApiImplicitParam(name = "orderQuery", value = "订单请求", paramType = "body", dataType = "OrderDetailQuery")
    public BaseResponse newOrder(@ApiParam @RequestBody OrderDetailQuery orderQuery) {

        orderQuery.setStatus(OrderStatus.WAIT_PAY.name());
        return iOrderService.createOrder(orderQuery);
    }

    @PostMapping("/new/entry")
    @ApiOperation("挂单")
    @ApiImplicitParam(name = "orderQuery", value = "订单请求", paramType = "body", dataType = "OrderQuery")
    public BaseResponse newEntryOrder(@RequestBody OrderDetailQuery orderQuery) {

        orderQuery.setStatus(OrderStatus.ENTRY_ORDER.name());

        return iOrderService.createOrder(orderQuery);
    }


    @PutMapping("/entry/")
    @ApiOperation("修改挂单信息")
    @ApiImplicitParam(name = "orderQuery", value = "订单请求", paramType = "body", dataType = "OrderQuery")
    public BaseResponse editEntryOrder(@RequestBody OrderDetailQuery orderQuery) {

        orderQuery.setStatus(OrderStatus.ENTRY_ORDER.name());
        return iOrderService.editOrder(orderQuery);
    }

    @GetMapping("list/user/{userId}/{status}")
    @ApiOperation("查询用户订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 支付中,2 待评论,3 订单完成 ,5 已退款,6 取消订单,  8 挂单 12 待发货 13 待收货 ,其他:查询所有]",
                    paramType = "path", dataType = "int"
            )
    })
    public BaseResponse listUsers(@PathVariable("userId") int userId, @PathVariable("status") int status) {

        BaseResponse responseEntity;
        Map<String, Object> condition = new HashMap<>();
        condition.put("user_id", userId);

        OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
        condition.put("order_status", orderStatus);
        responseEntity = iOrderService.getOrders(condition);
        return responseEntity;
    }

    @GetMapping("page/user/{userId}/{status}")
    @ApiOperation("查询用户订单  分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 确定认支付,2 待评论,3 订单完成,4 申请退款,5 已退款,6 取消订单 ,7 待售后 8 挂单 ,其他:查询所有]",
                    paramType = "path", dataType = "int"
            )
    })
    public BaseResponse pageUsers(PageQuery query, @PathVariable("userId") int userId, @PathVariable("status") int status) {

        BaseResponse responseEntity;
        Map<String, Object> condition = new HashMap<>();
        condition.put("user_id", userId);

        OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
        condition.put("order_status", orderStatus);
        responseEntity = iOrderService.getOrdersPage(condition, query);
        return responseEntity;
    }

    @GetMapping("list/seller/{seller}/{status}")
    @ApiOperation("查询商户订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "seller", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 支付中,2 待评论,3 订单完成 ,5 已退款,6 取消订单,  8 挂单 12 待发货 13 待收货 ,其他:查询所有]",
                    paramType = "path", dataType = "int"
            )
    })
    public BaseResponse listSeller(@PathVariable("seller") int seller, @PathVariable("status") int status) {

        BaseResponse responseEntity;
        Map<String, Object> condition = new HashMap<>();
        condition.put("seller_id", seller);
        OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
        condition.put("order_status", orderStatus);
        responseEntity = iOrderService.getOrders(condition);
        return responseEntity;
    }

    @PutMapping("/confirmPayment")
    @ApiOperation("确定付款(现金)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "payPrice", value = "实付金额", paramType = "query", dataType = "Double")
    })
    public BaseResponse confirmPayment(@NotNull(message = "订单号不能为空") String orderNo,
                                       @Min(value = 0, message = "实际支付金额需大于0") Double payPrice) {
        OrderDetailQuery query = new OrderDetailQuery();
        query.setOrderNo(orderNo);
        query.setPaas("现金支付");
        query.setPayment("现金支付");
        query.setPayPrice(payPrice);
        query.setStatus(OrderStatus.COMPLETE.name());
        query.setPayStatus(PayStatus.PAY_YES.name());

        BaseResponse response = iOrderService.editOrder(query);

        if (response.getCode() == 200) {
            Map condition = new HashMap();
            condition.put("order_no", orderNo);
            OrderResponse orderResponse = ordersService.getOneOrder(JSON.toJSONString(condition));
            return iCashPointRecordService.getPoints(orderNo, new BigDecimal(payPrice), orderResponse.getData().getUserId());
        } else {
            return response;
        }
    }

    @PutMapping("/scan/point")
    @ApiOperation("扫码获取积分")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id积分兑换", paramType = "query", dataType = "Long"),
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "query", dataType = "Long")
    })
    public BaseResponse<String> usePoint(Long id, Long userId) {
        return iCashPointRecordService.usePoint(id, userId);
    }

    @PutMapping("{orderNo}")
    @ApiOperation("支付订单")
    @ApiImplicitParam(name = "orderNo", value = "订单号儿", paramType = "path", dataType = "String")
    public BaseResponse payOrder(@PathVariable("orderNo") String orderNo) {

        return iOrderService.payOrder(orderNo);
    }

    @PutMapping("/after/success/{orderNo}")
    @ApiOperation("支付成功后的回调")
    public BaseResponse afterSuccess(@PathVariable("orderNo") String orderNo) {
        BaseResponse response = iOrderService.afterSuccess(orderNo);
        if (response.code == 200) {
            Map map = new HashMap();
            map.put("order_no", orderNo);
            OrderResponse orderResponse = ordersService.getOneOrder(JSON.toJSONString(map));
            Orders orders = orderResponse.getData();
            // 佣金推送
            if (orders != null && orderResponse.getCode() == 200) {
                System.out.println(orderNo + "订单状态改变：" + orderResponse.code + "|" + orders.getOrderStatus());
                if (OrderStatus.WAIT_COMMENT.name().equals(orders.getOrderStatus())) {
                    merchantProfitService.addMoneyData(orders.getSellerId(),
                            orders.getId(), orders.getPaymentMethodName(), orders.getPayPrice(), orders.getCreateTime());
                }
            }
        }
        return response;
    }

    @PutMapping("/after/fail/{orderNo}")
    @ApiOperation("支付失败后的回调")
    public BaseResponse afterFail(@PathVariable("orderNo") String orderNo) {
        return iOrderService.afterFail(orderNo);
    }

    @GetMapping("/orderNo")
    @ApiOperation("获取订单号")
    public BaseResponse getOrderNo() {

        return BaseResponse.ok(iOrderService.getPreOrderNo());
    }

    @GetMapping("/page/v2/seller/{seller}/{status}")
    @ApiOperation("查询商户订单  分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "seller", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 支付中,2 待评论,3 订单完成 ,5 已退款,6 取消订单,  8 挂单 12 待发货 13 待收货 ,其他:查询所有]",
                    paramType = "path", dataType = "int"
            ),
            @ApiImplicitParam(name = "start", value = "起始时间", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间", paramType = "query", dataType = "long")
    })
    public BaseResponse pageSeller(@PathVariable("seller") int seller, @PathVariable("status") int status,
                                   long start, long end, PageQuery pageQuery) {

        return iOrderService.pageSeller(seller, status, start, end, pageQuery);
    }

    @GetMapping("/page/seller/{seller}/{status}")
    @ApiOperation("查询商户订单  分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "seller", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 支付中,2 待评论,3 订单完成 ,5 已退款,6 取消订单,  8 挂单 12 待发货 13 待收货 ,其他:查询所有]",
                    paramType = "path", dataType = "int"
            )
    })
    public BaseResponse pageSeller(PageQuery query, @PathVariable("seller") int seller, @PathVariable("status") int status) {

        BaseResponse responseEntity;
        Map<String, Object> condition = new HashMap<>();
        condition.put("seller_id", seller);
        OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
        condition.put("order_status", orderStatus);
        responseEntity = iOrderService.getOrdersPage(condition, query);
        return responseEntity;
    }

    @DeleteMapping("{orderNo}")
    @ApiOperation("取消订单")
    @ApiImplicitParam(name = "orderNo", value = "订单号儿", paramType = "path", dataType = "String")
    public BaseResponse delOrder(@PathVariable("orderNo") String orderNo) {
        return iOrderService.cancelOrder(orderNo);
    }

    @PostMapping("/applyForRefund")
    public BaseResponse refund(String orderNo) {
        if (StrUtil.isEmpty(orderNo)) {
            return BaseResponse.error("订单号为空", "cashier");
        }
        Snowflake snowflake = IdUtil.createSnowflake(1, 1);
        String refundNo = snowflake.nextIdStr();
        Map<String, Object> condition = new HashMap<>(1);
        condition.put("order_no", orderNo);
        condition.put("pay_status", PayStatus.PAY_YES.name());
        OrderResponse response = ordersService.getOneOrder(JSON.toJSONString(condition));
        if (response.getCode() == 200) {
            Orders order = response.getData();
            BaseResponse response1 = izhPaymentService.applyForRefund(refundNo, orderNo, order.getPayPrice().multiply(new BigDecimal(100)).intValue() + "", order.getPayPrice().multiply(new BigDecimal(100)).intValue() + "", "");
            return response1;
        } else {
            return response;
        }
    }

    /**
     * @param start    上班时间
     * @param end      下班时间
     * @param sellerId 商户id
     * @return
     */
    @GetMapping("/mail/earning")
    @ApiOperation("发送当前班次收入统计")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "start", value = "上班时间", paramType = "query", dataType = "Long"),
            @ApiImplicitParam(name = "end", value = "下班时间", paramType = "query", dataType = "Long"),
            @ApiImplicitParam(name = "sellerId", value = "商户id", paramType = "query", dataType = "Long")
    })
    public BaseResponse sendMailEarningsAfterWork(Long start, Long end, Long sellerId) {
        return iOrderService.sendMailEarningsAfterWork(start, end, sellerId);
    }
}
