package cn.wise.sc.pay.task.timer.order.handler;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.api.order.service.IOrderCallExService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.order.model.OrderCallException;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import cn.wise.sc.pay.task.timer.order.model.PayRtsQueryParam;
import cn.wise.sc.pay.task.timer.order.model.PayRtsTradeState;
import cn.wise.sc.payment.api.service.IZHPaymentService;
import com.alibaba.fastjson.JSON;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @description: 订单调用招行支付回调异常 发起主动查询 定时处理器
 * @author: qh
 * @create: 2020-01-13 16:51
 **/
@Component
public class OrderPayCallExHandler {

    @Autowired
    IOrderCallExService orderCallExService;
    @Autowired
    IZHPaymentService izhPaymentService;
    @Autowired
    IOrdersService orderService;

    /**
     * 轮询支付失败的订单  主动发起支付结果查询
     */
    @XxlJob("OrderCallExceptionHandler")
    public ReturnT<String> execute(String s) {

        BaseResponse orderResponse = orderCallExService.getList(0);
        if (orderResponse.getCode() == 400) {
            return ReturnT.FAIL;
        }
        List<HashMap> hashMaps = orderResponse.getList();
        //HashMap转Bean
        List<OrderCallException> payExceptionList = hashMaps.stream()
                .map(arg -> BeanUtil.mapToBean(arg, OrderCallException.class, true))
                .collect(Collectors.toList());
        /*
         * 轮询  支付结果返回异常的订单，发起主动查询
         *       成功->修改订单状态为待发货,关闭支付异常状态
         *       失败->修改订单为已取消,关闭支付异常状态
         *       交易进行->不处理,失败处理次数+1
         *       转入退款-->修改订单状态为退款,关闭异常状态
         * */
        List<OrderDetailQuery> editOrders = new ArrayList<>();
        List<OrderCallException> editOrderCallExList = new ArrayList<>(payExceptionList.size());

        for (OrderCallException orderCallEx : payExceptionList) {
            BaseResponse payResponse = izhPaymentService.payResultQuery("", orderCallEx.getOrderNo());
                    OrderCallException editOrderCallEx = new OrderCallException();
                    BeanUtil.copyProperties(orderCallEx, editOrderCallEx);
                    if (payResponse.getCode() == 200) {
                        PayRtsQueryParam payRtsQueryParam = JSON.parseObject(payResponse.getData().toString(), PayRtsQueryParam.class);
                        OrderDetailQuery editOrder = new OrderDetailQuery();
                        editOrder.setUnifiedOrderNo(orderCallEx.getOrderNo());
                        if (PayRtsTradeState.S.name().equals(payRtsQueryParam.getTradeState())) {
                            editOrder.setStatus(OrderStatus.WAIT_DELIVERING.name());
                            editOrders.add(editOrder);
                            editOrderCallEx.setStatus(1);
                        } else if (PayRtsTradeState.P.name().equals(payRtsQueryParam.getTradeState())) {
                            editOrderCallEx.setFailCount(orderCallEx.getFailCount() + 1);
                        }  else {
                            editOrder.setStatus(OrderStatus.CANCELLED.name());
                            editOrders.add(editOrder);
                    editOrderCallEx.setStatus(1);
                }
            }
            editOrderCallExList.add(editOrderCallEx);
        }

        BaseResponse baseResponse = BaseResponse.ok("");

        if (editOrderCallExList.size() != 0) {
            baseResponse = orderCallExService.editBatchById(editOrderCallExList);
        }
        if (editOrders.size() != 0 && baseResponse.getCode() == 200) {
            baseResponse = orderService.updateBatch(editOrders);
        }
        if (baseResponse.getCode() == 200) {
            return ReturnT.SUCCESS;
        } else {
            return ReturnT.FAIL;
        }
    }
}
